#include <stdio.h>
#include <math.h>
#include "esp_log.h"
#include "spi_bus.h"
#include "screen_driver.h"
#include "lvgl_gui.h"
#include "display.h"
#include "main_menu.h"
static const char *TAG = "DISPLAY";
static const size_t queue_len = 10;
static scr_driver_t g_lcd;
static scr_info_t g_lcd_info;
static QueueHandle_t queue;
static lv_obj_t *north_needle;
static lv_obj_t *south_needle;
static float theta = 0;
static lv_point_t north_needle_points[] = {
		{ COMPASS_CENTER_X, COMPASS_CENTER_Y }, { 0, 0 } };
static lv_point_t south_needle_points[] = {
		{ COMPASS_CENTER_X, COMPASS_CENTER_Y }, { 0, 0 } };

static lv_obj_t *rx_label, *ry_label, *rz_label, *x_label, *y_label, *z_label,
		*theta_arc_label, *theta_deg_label;
static FP_MENU_PANEL current_menu_panel;
static void draw_direction();
static void init_screen() {
	esp_err_t ret = ESP_OK;
	spi_config_t bus_conf = { .miso_io_num = SPI2_MISO,
			.mosi_io_num = SPI2_MOSI, .sclk_io_num = SPI2_SCK,
			.max_transfer_sz = 2 * 240 * 240 + 10, };
	spi_bus_handle_t s_spi2_bus_handle = spi_bus_create(SPI2_HOST, &bus_conf);
	scr_interface_spi_config_t spi_lcd_cfg = { .spi_bus = s_spi2_bus_handle,
			.pin_num_cs = LCD_SPI_CS_PIN, .pin_num_dc = LCD_SPI_DC_PIN,
			.clk_freq = LCD_SPI_CLOCK_FREQ, .swap_data = true, };
	scr_interface_driver_t *iface_drv;
	scr_interface_create(SCREEN_IFACE_SPI, &spi_lcd_cfg, &iface_drv);
	ret = scr_find_driver(SCREEN_CONTROLLER_ST7789, &g_lcd);
	if (ESP_OK != ret) {
		ESP_LOGE(TAG, "screen find failed");
		return;
	}
	scr_controller_config_t lcd_cfg =
			{ .interface_drv = iface_drv, .pin_num_rst = LCD_SPI_RESET_PIN,
					.pin_num_bckl = LCD_SPI_BL_PIN, .rst_active_level = 0,
					.bckl_active_level = 1, .offset_hor = 0, .offset_ver = 0,
					.width = 240, .height = 240, .rotate = SCR_DIR_LRTB, };
	ret = g_lcd.init(&lcd_cfg);
	if (ESP_OK != ret) {
		ESP_LOGE(TAG, "screen initialize failed");
		return;
	}
	g_lcd.get_info(&g_lcd_info);
	ESP_LOGD(TAG, "Screen name:%s | width:%d | height:%d", g_lcd_info.name,
			g_lcd_info.width, g_lcd_info.height);
	lvgl_init(&g_lcd, NULL);
}
static void init_menu() {
	init_main_menu();
	current_menu_panel = main_menu_panel;
	current_menu_panel(SHOW_PANEL, MAINTAIN_ITEM);
}
void init_display(QueueHandle_t *const ref_queue) {
	queue = xQueueCreate(queue_len, sizeof(DisplayMsg));
	*ref_queue = queue;
	init_screen();
	create_compass();
	init_menu();
}
static void process_compass_msg(const DisplayMsg *const msg) {
	char buffer[20] = "";
	theta = msg->theta;
	ESP_LOGD(TAG, "Theta arc:%f, degree:%f", msg->theta, msg->theta/M_PI*180);
	sprintf(buffer, "RAW X: %d", msg->rx);
	lv_label_set_text(rx_label, buffer);
	sprintf(buffer, "RAW Y: %d", msg->ry);
	lv_label_set_text(ry_label, buffer);
	sprintf(buffer, "RAW Z: %d", msg->rz);
	lv_label_set_text(rz_label, buffer);
	sprintf(buffer, "X: %.2f", msg->x);
	lv_label_set_text(x_label, buffer);
	sprintf(buffer, "Y: %.2f", msg->y);
	lv_label_set_text(y_label, buffer);
	sprintf(buffer, "Z: %.2f", msg->z);
	lv_label_set_text(z_label, buffer);
	sprintf(buffer, "Arc: %.2f", msg->theta);
	lv_label_set_text(theta_arc_label, buffer);
	sprintf(buffer, "Deg: %.2f", msg->theta / M_PI * 180);
	lv_label_set_text(theta_deg_label, buffer);
	draw_direction();
}
static void process_menu_navigate_msg(DisplayMsg *msg) {
	ESP_LOGI(TAG,"pointer:%p",current_menu_panel);
//	current_menu_panel(msg->page_action, msg->item_action);
}
void display_task(void *args) {
	DisplayMsg msg;
	while (1) {
		ESP_LOGI(TAG,"pointer:%p",current_menu_panel);
		if (xQueueReceive(queue, &msg, 100) == pdTRUE) {
			if (msg.type == TYPE_COMPASS_READ)
				process_compass_msg(&msg);
			else if (msg.type == TYPE_MENU_NAVIGATE) {
				process_menu_navigate_msg(&msg);
			}
		}
	}
}
void create_compass_label() {
	uint16_t compass_size = COMPASS_RADIUS * 2;
	lv_obj_t *x_label = lv_label_create(lv_scr_act(), NULL);
	lv_label_set_text(x_label, "X");
	lv_obj_align(x_label, NULL, LV_ALIGN_IN_TOP_LEFT,
			COMPASS_PADDING + compass_size, compass_size / 2 + COMPASS_PADDING);
	lv_obj_t *y_label = lv_label_create(lv_scr_act(), NULL);
	lv_label_set_text(y_label, "Y");
	lv_obj_align(y_label, NULL, LV_ALIGN_IN_TOP_LEFT,
			COMPASS_PADDING + compass_size / 2, 0);
}
void create_compass() {
	lv_obj_t *arc = lv_arc_create(lv_scr_act(), NULL);
	lv_arc_set_bg_angles(arc, 0, 360);
	lv_coord_t size = COMPASS_RADIUS * 2;
	lv_obj_set_size(arc, size, size);
	lv_obj_align(arc, NULL, LV_ALIGN_IN_TOP_LEFT, COMPASS_PADDING,
	COMPASS_PADDING);
	lv_obj_set_style_local_border_width(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT,
			0);
	lv_obj_set_style_local_line_width(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 1);
	lv_obj_set_style_local_pad_top(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
	lv_obj_set_style_local_pad_left(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
	lv_obj_set_style_local_pad_right(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
	lv_obj_set_style_local_pad_bottom(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
	lv_obj_set_style_local_bg_opa(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT,
			LV_OPA_TRANSP);

	static lv_point_t x_points[] = { { COMPASS_CENTER_X, COMPASS_CENTER_Y }, {
	COMPASS_RADIUS + COMPASS_CENTER_X, COMPASS_CENTER_Y } };
	static lv_point_t y_points[] = { { COMPASS_CENTER_X, COMPASS_CENTER_Y }, {
	COMPASS_CENTER_X, 0 } };
	lv_obj_t *x_line = lv_line_create(arc, NULL);
	lv_obj_set_style_local_line_width(x_line, LV_LINE_PART_MAIN,
			LV_STATE_DEFAULT, 1);
	lv_line_set_points(x_line, x_points, 2);

	lv_obj_t *y_line = lv_line_create(arc, NULL);
	lv_obj_set_style_local_line_width(y_line, LV_LINE_PART_MAIN,
			LV_STATE_DEFAULT, 2);
	lv_line_set_points(y_line, y_points, 2);

	north_needle = lv_line_create(arc, NULL);
	lv_obj_set_style_local_line_color(north_needle, LV_LINE_PART_MAIN,
			LV_STATE_DEFAULT,
			LV_COLOR_RED);
	lv_obj_set_style_local_line_width(north_needle, LV_LINE_PART_MAIN,
			LV_STATE_DEFAULT, 5);

	south_needle = lv_line_create(arc, NULL);
	lv_obj_set_style_local_line_color(south_needle, LV_LINE_PART_MAIN,
			LV_STATE_DEFAULT,
			LV_COLOR_BLUE);
	lv_obj_set_style_local_line_width(south_needle, LV_LINE_PART_MAIN,
			LV_STATE_DEFAULT, 5);
	vTaskDelay(pdMS_TO_TICKS(100));
	create_compass_label();
	create_reading_label();
}
void create_reading_label() {
	char buffer[15] = "";
	rx_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "RAW X:%u", 0);
	lv_label_set_text(rx_label, buffer);
	lv_obj_align(rx_label, NULL, LV_ALIGN_IN_TOP_LEFT, COMPASS_PADDING, 110);
	vTaskDelay(pdMS_TO_TICKS(100));

	ry_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "RAW Y:%u", 0);
	lv_label_set_text(ry_label, buffer);
	lv_obj_align(ry_label, NULL, LV_ALIGN_IN_TOP_LEFT, COMPASS_PADDING, 130);
	vTaskDelay(pdMS_TO_TICKS(100));

	rz_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "RAW Z:%u", 0);
	lv_label_set_text(rz_label, buffer);
	lv_obj_align(rz_label, NULL, LV_ALIGN_IN_TOP_LEFT, COMPASS_PADDING, 150);
	vTaskDelay(pdMS_TO_TICKS(100));

	x_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "X:%d", 0);
	lv_label_set_text(x_label, buffer);
	lv_obj_align(x_label, NULL, LV_ALIGN_IN_TOP_LEFT, COMPASS_PADDING, 180);
	vTaskDelay(pdMS_TO_TICKS(100));

	y_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "Y:%d", 0);
	lv_label_set_text(y_label, buffer);
	lv_obj_align(y_label, NULL, LV_ALIGN_IN_TOP_LEFT, COMPASS_PADDING, 200);
	vTaskDelay(pdMS_TO_TICKS(100));

	z_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "Z:%d", 0);
	lv_label_set_text(z_label, buffer);
	lv_obj_align(z_label, NULL, LV_ALIGN_IN_TOP_LEFT, COMPASS_PADDING, 220);
	vTaskDelay(pdMS_TO_TICKS(100));

	theta_arc_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "Arc:%.3f", 0.0f);
	lv_label_set_text(theta_arc_label, buffer);
	lv_obj_align(theta_arc_label, NULL, LV_ALIGN_IN_TOP_LEFT, 140, 25);
	vTaskDelay(pdMS_TO_TICKS(100));

	theta_deg_label = lv_label_create(lv_scr_act(), NULL);
	sprintf(buffer, "Deg:%.3f", 0.0f);
	lv_obj_align(theta_deg_label, NULL, LV_ALIGN_IN_TOP_LEFT, 140, 65);
	lv_label_set_text(theta_deg_label, buffer);
}
void compute_end_point() {
	north_needle_points[1].x = NEEDLE_LENGTH * cos(theta) + COMPASS_CENTER_X;
	north_needle_points[1].y = NEEDLE_LENGTH * sin(theta) + COMPASS_CENTER_Y;
	south_needle_points[1].x = -NEEDLE_LENGTH * cos(theta) + COMPASS_CENTER_X;
	south_needle_points[1].y = -NEEDLE_LENGTH * sin(theta) + COMPASS_CENTER_Y;
}
static void draw_direction() {
	compute_end_point();
	lv_line_set_points(north_needle, north_needle_points, 2);
	lv_line_set_points(south_needle, south_needle_points, 2);
}

