#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"
#include "driver/gpio.h"
#include "inputs.h"
#include "esp_log.h"
#include "esp_adc_cal.h"
#include "driver/adc.h"
#include "soc/adc_channel.h"
static FP_INPUT handler;
static const char *TAG = "INPUT";
static const adc_channel_t x_channel = ADC1_GPIO35_CHANNEL;
static const adc_channel_t y_channel = ADC1_GPIO34_CHANNEL;
static const adc_bits_width_t width = ADC_WIDTH_BIT_12;
static const adc_atten_t atten = ADC_ATTEN_DB_11;
static esp_adc_cal_characteristics_t *adc_chars;
static QueueHandle_t queue;
static int64_t last_key_pressed_on = 0;
static uint8_t press_count = 0;
static const uint16_t SPAN_BETWEEN_2_CLICKS = 500;
static void IRAM_ATTR gpio_isr_handler(void *arg) {
	uint8_t act = KEY_PRESS;
	int64_t current = esp_timer_get_time();
	press_count++;
	press_count %= 2;
	bool ignore = current - last_key_pressed_on <= SPAN_BETWEEN_2_CLICKS || !press_count;
	last_key_pressed_on = current;
	if (!ignore)
		xQueueSendFromISR(queue, &act, NULL);
}
static void print_char_val_type(esp_adc_cal_value_t val_type) {
	if (val_type == ESP_ADC_CAL_VAL_EFUSE_TP) {
		ESP_LOGD(TAG, "Characterized using Two Point Value\n");
	} else if (val_type == ESP_ADC_CAL_VAL_EFUSE_VREF) {
		ESP_LOGD(TAG, "Characterized using eFuse Vref\n");
	} else {
		ESP_LOGD(TAG, "Characterized using Default Vref\n");
	}
}
static void init_key() {
	gpio_config_t io_conf;
	io_conf.intr_type = GPIO_INTR_POSEDGE;
	io_conf.pin_bit_mask = GPIO_INPUT_KEY_SEL;
	io_conf.mode = GPIO_MODE_INPUT;
	io_conf.pull_up_en = 0;
	gpio_config(&io_conf);
	gpio_install_isr_service(ESP_INTR_FLAG_DEFAULT);
	gpio_isr_handler_add(K_IO, gpio_isr_handler, NULL);
}
static void init_joystick() {
	adc1_config_width(width);
	adc1_config_channel_atten(x_channel, atten);
	adc1_config_channel_atten(y_channel, atten);
	adc_chars = calloc(1, sizeof(esp_adc_cal_characteristics_t));
	esp_adc_cal_value_t val_type = esp_adc_cal_characterize(ADC_UNIT_1, atten,
			width,
			DEFAULT_VREF, adc_chars);
	print_char_val_type(val_type);
}
void init_inputs(FP_INPUT on_input_received) {
	queue = xQueueCreate(10, sizeof(uint8_t));
	handler = on_input_received;
	init_key();
	init_joystick();
}
static void queue_task(void *args) {
	uint8_t msg;
	while (1) {
		if (xQueueReceive(queue, &msg, 500) == pdTRUE) {
			handler(msg);
		}
	}
}
void input_task(void *args) {
	xTaskCreate(queue_task, "input_queue", 4096, NULL, 5, NULL);
	while (1) {
		uint32_t adc_x_reading = 0;
		uint32_t adc_y_reading = 0;
		for (int i = 0; i < NO_OF_SAMPLES; i++) {
			adc_x_reading += adc1_get_raw((adc1_channel_t) x_channel);
			adc_y_reading += adc1_get_raw((adc1_channel_t) y_channel);
		}
		adc_x_reading /= NO_OF_SAMPLES;
		adc_y_reading /= NO_OF_SAMPLES;
		uint32_t x_voltage = esp_adc_cal_raw_to_voltage(adc_x_reading,
				adc_chars);
		uint32_t y_voltage = esp_adc_cal_raw_to_voltage(adc_y_reading,
				adc_chars);
		uint8_t act = 0;
		if (x_voltage < 500)
			act = X_NEG;
		else if (x_voltage > 1700)
			act = X_POS;
		else if (y_voltage < 500)
			act = Y_NEG;
		else if (y_voltage > 2500)
			act = Y_POS;
		else
			act = UNDEFINED;
		if (act != UNDEFINED)
			xQueueSend(queue, &act, 0);
		vTaskDelay(pdMS_TO_TICKS(500));
		ESP_LOGD(TAG, "X:%d mV, Y:%d mV", x_voltage, y_voltage);
	}
}
